/*
	File name:	2dto2d-mainc
	Language:	C
	Type:		Main function
	Author:		Gang Liu
	Date:		9/12/98
	Function:	Contains the main function which takes arguments
			from the command line, calls
			_2dto2d_perspective_projection() to perform the
			desired 2D to 2D perspective projection of an input
			image, and save the resulting projected image into a
			disk file in PGM format.
*/
#include <stdio.h>
#include <stdlib.h>

#include "must.h"
#include "pgm.h"

#include "2dto2d.h"

void	usage(FILE *fp, char *cmd);

int	main(int argc, char *argv[])
{
	int	i;

	float	c0, r0, omega, phi, kappa, x0, y0, z0, f, delta_u, delta_v;
	char	*inname, *outname;

	uchar	**inimage, **outimage;
	int	inrows, incols, outrows, outcols;

	float	deg2rad=3.1415927/180.0;

	if(argc<14) {
		usage(stdout, argv[0]);
		return -1;
	}

	i=1;
	inname=argv[i++];
	c0=atof(argv[i++]);
	r0=atof(argv[i++]);
	omega=atof(argv[i++]);
	phi=atof(argv[i++]);
	kappa=atof(argv[i++]);
	x0=atof(argv[i++]);
	y0=atof(argv[i++]);
	z0=atof(argv[i++]);
	f=atof(argv[i++]);
	delta_u=atof(argv[i++]);
	delta_v=atof(argv[i++]);
	outname=argv[i++];

	omega *= deg2rad;
	phi *= deg2rad;
	kappa *= deg2rad;

	i=ReadPGM(inname, &incols, &inrows, &inimage);
	if(i) myerror(i, stderr, "ReadPGM() failed.");

	i=_2dto2d_perspective_projection(inimage, inrows, incols, c0, r0,
		omega, phi, kappa, x0, y0, z0, f, delta_u, delta_v,
		&outimage, &outrows, &outcols);
	if(i) myerror(i, stderr, "_2dto2d_perspective_projection() failed.");

	i=WritePGM(outname, outcols, outrows, outimage);
	if(i) myerror(i, stderr, "WritePGM() failed.");

	free2d((void**)inimage);
	free2d((void**)outimage);

	return 0;
}


void	usage(FILE *fp, char *cmd)
{
fprintf(fp,
"Usage:\n\t%s <F1> <c0> <r0> <omega> <phi> <kappa> <x0> <y0> <z0> <f> <du> <dv> <F2>\n",
	cmd);
fprintf(fp,
"where\n\
\t<F1>\t is the name of the PGM file for the input image;\n\
\t<c0>\t is the column coordinate in the image of the object frame;\n\
\t<r0>\t is the row coordinate in the image of the object frame;\n\
\t<omega>\t is the rotation angle in degrees around the X axis;\n\
\t<phi>\t is the rotation angle in degrees around the Y axis;\n\
\t<kappa>\t is the rotation angle in degrees around the Z axis;\n\
\t<x0>\t is the X coordinate of the origin of the object frame;\n\
\t<y0>\t is the Y coordinate of the origin of the object frame;\n\
\t<z0>\t is the Z coordinate of the origin of the object frame;\n\
\t<f>\t is the focal length;\n\
\t<du>\t is the pixel size along the u axis;\n\
\t<dv>\t is the pixel size along the v axis;\n\
\t<F2>\t is the name of the PGM file for the output image;\n\
");
fprintf(fp,
"\nNote:\n\
	The image is attached to the a-b plane of the object frame, with the\n\
	column-increasing direction aligned with the a axis of the frame, and\n\
	the row-increasing direction aligned with the b axis. The image is\n\
	placed on the a-b plane in such direction, and with the origin of\n\
	the a-b plane located at (row,column)=(r0,c0) in the image.\n\
\n\
	The orientation of the object frame relative to the camera frame is\n\
	specified by first rotating the camera frame about its x axis for\n\
	the amount of 'omega', the rotating the rotated frame about its y\n\
	axis for 'phi', and finally rotating the twice rotated frame about\n\
	its z axis for 'kappa'. Then the origin of the last frame is moved to\n\
	(x0,y0,z0) of the camera frame, resulting in the final object frame.\n\
\n\
	The focal length, or the location of the imaging plane on the z axis\n\
	in the camera frame, is given as 'f', and the digitization on the\n\
	imaging plane is determined by the pixel size specified by 'delta_u'\n\
	and 'delta_v', for the u and v axes, respectively.\n\
");
}
