#include <stdio.h>	/* for NULL */
#include <malloc.h>
#include <memory.h>
#include "boundary.h"

/* the nine-th element is for the first node of the chain */
static	const	point
	ChainDir[9]= {	{1,0},{1,-1},{0,-1},{-1,-1},
			{-1,0},{-1,1},{0,1},{1,1},  {0,0}};

/*
 * global data (static to this file) used by BoundaryFill4(), which is
 * the kernel of boundary_fill(), to improve performance by reducing
 * unnecessary stack usage.
 */
static	color	**BoundaryFill4_map;
static	int	BoundaryFill4_width, BoundaryFill4_height;

static void	BoundaryFill4(int x, int y, color bv, color nv);
static int	get_dir_code(point p);

int	find_boundary(color **image, int width, int height, color interest,
		int connectivity, ChainCode *bound)
{
static	const	point
	neighbours[8]={{-1,0},{-1,-1},{0,-1},{1,-1},{1,0},{1,1},{0,1},{-1,1}};

	point	current, last, D;
	int	b;
	int	new, step;

	int	i, j;
	int	x, y;
	int	found;
	long	len;
	char	*tmp_chain;

	if(connectivity!=4 && connectivity!=8) connectivity=8;
	step=(connectivity==8) ? 1 : 2;

	tmp_chain=(char*)malloc((long)2*width*height*sizeof(char));
	if(tmp_chain==NULL) return -1;
	len = 0;

	bound->y = -1;
	bound->x = -1;
	bound->len = 0;
	bound->chain = NULL;

	found=0;
	for(i=0;i<height && !found;i++)
		for(j=0;j<width;j++)
			if( image[i][j]==interest ) {
				current.y = i;
				current.x = j;
				found=1;
				break;
			}
	if( !found ) {
		free(tmp_chain);
		return 0; /* no pixel in the image has the interested label */
	}

	bound->y = current.y;
	bound->x = current.x;

	b=0;
	new=1;
	last = current;
	while( new ) {
		D.y = current.y - last.y;
		D.x = current.x - last.x;
		i=get_dir_code(D);
		if(i<0 || i>8) {
			printf("Internal error:get_dir_code()\n");
			exit(-1);
		}
		tmp_chain[ len++ ] = i;
		last = current;

		new = 0;
		for(i=0;i<connectivity;i++, b=(b+step)%8) {
			y = current.y + neighbours[b].y;
			x = current.x + neighbours[b].x;
			if( x<0 || x>=width || y<0 || y>=height ) continue;
			if( y==bound->y && x==bound->x )
				break; /* back to the beginning of the chain */
			if( image[y][x] != interest ) continue;

			current.y = y; current.x = x;
			switch(b) {
				case 0:
				case 1:	b=6; break;
				case 2:
				case 3:	b=0; break;
				case 4:
				case 5:	b=2; break;
				case 6:
				case 7:	b=4; break;
				default:
					printf("Never should happen!\n");
					exit(-1);
			}
			new = 1;
			break;
		}
	}

	bound->len = len;
	bound->chain = (char*)malloc(len*sizeof(char));
	if(bound->chain!=NULL)
		memcpy(bound->chain, tmp_chain, len*sizeof(char));
	free(tmp_chain);

	if(bound->chain==NULL) return -1;
	return 0;
}

void	free_chain(ChainCode *chain)
{
	free(chain->chain);
	return;
}

int	chain2list(ChainCode *chain, point **list, int *len)
{
	int	x, y;
	int	k;

	*len = chain->len;
	if( (*len) <= 0 ) {
		*list = NULL;
		return 0;
	}

	*list=(point*)malloc((*len)*sizeof(point));
	if(*list==NULL) return -1;

	y = chain->y;
	x = chain->x;
	for(k=0;k<(*len);k++) {
		y += ChainDir[ (int)(chain->chain[k]) ].y;
		x += ChainDir[ (int)(chain->chain[k]) ].x;
		(*list)[k].x = x;
		(*list)[k].y = y;
	}
	return 0;
}

void	draw_boundary(color **image, int width, int height, color interest,
		ChainCode *bound)
{
	int	x, y;
	int	k;

	y = bound->y;
	x = bound->x;
	for(k=0;k<bound->len;k++) {
		y += ChainDir[ (int)(bound->chain[k]) ].y;
		x += ChainDir[ (int)(bound->chain[k]) ].x;
		if( x<0 || x>=width || y<0 || y>=height ) continue;
		image[y][x] = interest;
	}
	return;
}

static int	get_dir_code(point p)
{
	int	val;
/*
ChainDir[9]= {	{1,0},{1,-1},{0,-1},{-1,-1},
		{-1,0},{-1,1},{0,1},{1,1},  {0,0}};
*/
	switch(p.y) {
	case -1:
		switch(p.x) {
		case -1:	val = 3; break;
		case  0:	val = 2; break;
		case  1:	val = 1; break;
		default:	val = -1; break;
		}
		break;
	case  0:
		switch(p.x) {
		case -1:	val = 4; break;
		case  0:	val = 8; break;
		case  1:	val = 0; break;
		default:	val = -1; break;
		}
		break;
	case  1:
		switch(p.x) {
		case -1:	val = 5; break;
		case  0:	val = 6; break;
		case  1:	val = 7; break;
		default:	val = -1; break;
		}
		break;
	default:	val = -1; break;
	}

	return val;
}

void	boundary_fill(color **map, int width, int height,
			int x, int y, color bv, color nv)
{
extern	color	**BoundaryFill4_map;
extern	int	BoundaryFill4_width, BoundaryFill4_height;

	BoundaryFill4_map	= map;
	BoundaryFill4_width	= width;
	BoundaryFill4_height	= height;

	BoundaryFill4(x, y, bv, nv);
	return;
}

static void	BoundaryFill4(int x, int y, color bv, color nv)
{
extern	color	**BoundaryFill4_map;
extern	int	BoundaryFill4_width, BoundaryFill4_height;

	color c;

	if(x<0 || x>=BoundaryFill4_width || y<0 || y>=BoundaryFill4_height)
		return;

	c=BoundaryFill4_map[y][x];
	if( c != bv &&		/* Not yet at the boundary ... */
	    c != nv ) {		/* Nor have we been here before ... */
		BoundaryFill4_map[y][x] = nv;
		BoundaryFill4(x, y-1, bv, nv);
		BoundaryFill4(x, y+1, bv, nv);
		BoundaryFill4(x-1, y, bv, nv);
		BoundaryFill4(x+1, y, bv, nv);
	}
	return;
}	/* BoundaryFill4 */
