/*
	File name:	editpgm.c
	Language:	C
	Author:		Gang Liu
	Date:		10/19/97
	Type:		main program
	Function:	Given an image and a series of triples of (x,y,v),
			change the pixel value at (x,y) to v.

*/
#include <stdio.h>
#include <malloc.h>
#include <stdlib.h>

#include "must.h"
#include "pgm.h"

void	usage(FILE *fp, char *cmd);

int	main(int argc, char *argv[])
{
	int	i, k;

	int	inputs;
	char	*inname, *outname;

	uchar	**image;
	int	width, height;

	int	x, y, v;

	if(argc < 6) {
		usage(stderr, argv[0]);
		return -1;
	}

	inname = argv[1];
	outname = argv[argc-1];
	inputs = (argc-3) / 3;	/* less command name, input and output names */

	i=ReadPGM(inname, &width, &height, &image);
	if(i) myerror(i, stderr, "Error reading input image.\n");

	for(k=0;k<inputs;k++) {
		x= atoi( argv[2+k*3+0] );
		y= atoi( argv[2+k*3+1] );
		v= atoi( argv[2+k*3+2] );

		if(x<0 || x>=width || y<0 || y>=height) {
			fprintf(stderr,
			"(x,y)=(%d,%d) out of image, ignored.\n", x, y);
			continue;
		}
		if(v<0) v=0;
		if(v>255) v=255;

		image[y][x]=v;
	}


	i=WritePGM(outname, width, height, image);
	if(i) myerror(i, stderr, "Error writing output image.\n");

	free2d((void**)image);

	return 0;
}

void	usage(FILE *fp, char *cmd)
{
fprintf(fp,
"\nFunction of %s:\n\n\
Given an images and a series of triples (x,y,v), change the pixel value\n\
at (x,y) to v.\n\n", cmd);

fprintf(fp,
"Usage: %s <fi> <x1 y1 v1> [<x2 y2 v2> ...] <fo>\n\
where\n\
\t <fi> is the input image file (PGM format) name\n\
\t <x1 y1 v1> is the first triple\n\
\t [<x1 y1 v1> ...] optional more triples\n\
\t <fo> is the output image file (PGM format) name\n\
\n", cmd);
}
